@@ -747,20 +747,22 @@ static Linkage_info * linkage_info_new(int num_to_alloc)
 
 static void linkage_info_delete(Linkage_info *link_info, int sz)
 {
-#ifdef USE_CORPUS
 	int i,j;
 
 	for (i=0; i<sz; i++)
 	{
 		Linkage_info *lifo = &link_info[i];
-		for (j=0; j<MAX_SENTENCE; j++)
+		int nwords = lifo->nwords;
+		for (j=0; j<nwords; j++)
 		{
 			if (lifo->disjunct_list_str[j])
 				free(lifo->disjunct_list_str[j]);
-			lifo->disjunct_list_str[j] = NULL;
 		}
-	}
+		free(lifo->disjunct_list_str);
+#ifdef USE_CORPUS
+		lg_sense_delete(lifo);
 #endif
+	}
 	xfree(link_info, sz);
 }
 
@@ -1330,7 +1332,7 @@ static int link_already_appears(Linkage linkage, Link *link, int a)
 
 static PP_info excopy_pp_info(PP_info ppi)
 {
-	static PP_info newppi;
+	PP_info newppi;
 	int i;
 
 	newppi.num_domains = ppi.num_domains;
@@ -1528,7 +1530,7 @@ Sentence linkage_get_sentence(Linkage linkage)
 
 const char * linkage_get_disjunct(Linkage linkage, int w)
 {
-	if (NULL == linkage->info->disjunct_list_str[w])
+	if (NULL == linkage->info->disjunct_list_str)
 	{
 		lg_compute_disjunct_strings(linkage->sent, linkage->info);
 	}
